VecMut

Struct VecMut 

pub struct VecMut<'a, T> { /* private fields */ }
A reference to a single column/row of a matrix.

This is similar to &[T] but with a pitch potentially different from 1 between its elements, i.e. there is no guarantee of contiguity. As a consequence this does not have a simple past-the-end pointer like a slice would have. For an empty slice the only guaranteed-valid pointer is the base pointer itself while for larger slices the last guaranteed-valid pointer is one-past the last element, not one additional pitch.

Created from its constructors or a block reference via the BlockMut::col and BlockMut::row methods.

Implementations

Source

impl<'data, T> VecMut<'data, T>

pub fn new(data: &'data mut [T], pitch: usize) -> Self

Create a new vector reference from a raw slice and pitch.

The resulting block refers to the first column of the matrix.

pub fn from_slice(data: &'data mut [T]) -> Self

Create a new vector reference from a raw slice with pitch 1.

pub fn len(&self) -> usize

Number of elements in this vector.

pub fn is_empty(&self) -> bool

Whether this vector is empty.

pub fn split_at(self, mid: usize) -> (VecMut<'data, T>, VecMut<'data, T>)

Divide into two vectors at the given element.

Examples
use matrix_slice::VecMut;

let data = &mut [0, 1, 2, 3, 4, 5];

let block = VecMut::new(data, 1);
let (left, right) = block.split_at(2);

assert_eq!(left[1], 1);
assert_eq!(right[3], 5);
pub fn split_at_checked( self, mid: usize, ) -> Option<(VecMut<'data, T>, VecMut<'data, T>)>

Divide into two vectors at the given element.

See Self::split_at but returns None if out of bounds.

pub fn select<R>(self, range: R) -> Option<VecMut<'data, T>>
where R: MatrixIndex,

Choose a range of elements and contract the vector to that.

pub fn cast_const(self) -> VecRef<'data, T>

Turn this unique reference into a shared reference.

pub fn reborrow(&mut self) -> VecMut<'_, T>

Create a unique reference to this block with a shorter lifetime.

pub fn as_cells(self) -> VecMut<'data, Cell<T>>

Modify the item type to a Cell, allowing interior mutability.

This is the equivalent of Cell::from_mut over elements in this slice.

Source

impl<'data, T> VecMut<'data, Cell<T>>

pub fn as_cell_items(self) -> VecMut<'data, T>

Modify the item type from a Cell to its interior type.

This is the equivalent of Cell::get_mut over elements in this slice.